package be.seeseemelk.llamascript;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import java.util.HashMap;

@Slf4j
@RequiredArgsConstructor
public class LlamaScript {
    private final LlamaScriptWorker worker;

    public <T> T eval(String prompt, Class<T> returnType, Object... arguments) {
        var argumentMap = new HashMap<Integer, Object>();
        for (Object argument : arguments) {
            argumentMap.put(argumentMap.size(), argument);
        }

        if (returnType.equals(Integer.class)) {
            //noinspection unchecked
            return (T) evalInt(prompt, argumentMap);
        } else if (returnType.equals(String.class)) {
            //noinspection unchecked
            return (T) evalString(prompt, argumentMap);
        } else if (returnType.equals(Boolean.class)) {
            //noinspection unchecked
            return (T) evalBool(prompt, argumentMap);
        } else {
            throw new LLamaScriptException("Unsupported return type");
        }
    }

    public String evalString(String prompt, HashMap<Integer, Object> argumentMap) {
        var result = worker.eval(prompt, primitiveOutputType("a string"), argumentMap);
        if (result instanceof String string)
            return string;
        else if (result != null)
            return result.toString();
        else
            throw new LLamaScriptException("No response");
    }

    public String evalString(String prompt, Object... arguments) {
        return eval(prompt, String.class, arguments);
    }

    public Integer evalInt(String prompt, HashMap<Integer, Object> argumentMap) {
        var result = worker.eval(prompt, primitiveOutputType("an integer number"), argumentMap);
        if (result instanceof Integer integer)
            return integer;
        else if (result instanceof String string) {
            try {
                return Integer.parseInt(string);
            } catch (NumberFormatException e) {
                throw new LLamaScriptException("Invalid integer", e);
            }
        }
        else
            throw new LLamaScriptException("Invalid integer type");
    }

    public int evalInt(String prompt, Object... arguments) {
        return eval(prompt, Integer.class, arguments);
    }

    public Boolean evalBool(String prompt, HashMap<Integer, Object> argumentMap) {
        var result = worker.eval(prompt, primitiveOutputType("a boolean value"), argumentMap);
        if (result instanceof Boolean bool)
            return bool;
        else if (result instanceof String string)
            return Boolean.parseBoolean(string);
        else
            throw new LLamaScriptException("Invalid boolean type");
    }

    public boolean evalBool(String prompt, Object... arguments) {
        return eval(prompt, Boolean.class, arguments);
    }

    private static String primitiveOutputType(String description) {
        return "This result must contain %s.".formatted(description);
    }
}
